/**
 * Header file for project logging.
 */


// System Import Headers.
#include <cstdio>
#include <cstdlib>
#include <cctype>
#include <execinfo.h>
#include <iostream>
#include <filesystem>
#include <fstream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>


/**
 * Logging class definition.
 */
class Logging {

    private:
        std::string core_logging_folder;
        std::ofstream debug_stream;
        std::ofstream info_stream;
        std::ofstream warn_stream;
        std::ofstream error_stream;
        std::ofstream timer_stream;

    public:
        /**
         * Class constructor.
         */
        Logging();

        /**
         * Class deconstructor.
         * Closes file descriptors, if open.
         */
        ~Logging();

        /**
         * Closes all open logging file descriptors.
         */
        void close_log_files();

        /**
         * Writes message to info log file.
         */
        void debug(std::string msg, std::string log_type = "DEBUG", const char* file = __builtin_FILE(), long line_num  = __builtin_LINE());

        /**
         * Writes message to info log file.
         */
        void info(std::string msg, std::string log_type = "INFO", const char* file = __builtin_FILE(), long line_num  = __builtin_LINE());

        /**
         * Writes message to warning log file.
         */
        void warn(std::string msg, std::string log_type = "WARN", const char* file = __builtin_FILE(), long line_num  = __builtin_LINE());

        /**
         * Writes message to error log file.
         */
        void error(std::string msg, std::string log_type = "ERROR", const char* file = __builtin_FILE(), long line_num  = __builtin_LINE());

        /**
         * Writes message to timer log file.
         */
        void timer(std::string msg, std::string log_type = "TIMER", const char* file = __builtin_FILE(), long line_num  = __builtin_LINE());


        /**
         * Prints out logging location to console.
         */
        void display_logging_folder();

        /**
         * Resets contents of logging files.
         */
        void clear_log_files();
};
